// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
 dpscolfix

   Authors:  Aitor Ibarra - XMM-Newton SOC
         
   Description: Correct dps bit pattern for float number
 
   Comments: 
 
  $Id: module.cc,v 1.5 2000/10/02 08:30:13 gvacanti Exp $
*/

#include "dpscolfix.h"

#include <Tracer.h>
#include <errstr.h>
#include "Dal.h"
#include "dal_utilities.h"
#include <stdio.h>
#include <cstring>

using namespace std;

void dpscolfix ()
{
PROC

	string file = stringParameter("file");
	DataSet *s = dataSetServer->open(file, Dal::Modify);

	//Don't do anything if the keyword already exists
	if (!s->hasAttribute("DPSCORRF"))
	{


		//Do the trick!!!
		real64 dest=0.0;
		unsigned int tabs = s->blocks();

		//For all table in the file
		for (unsigned int t=0 ; t < tabs ; ++t)
		{
			Table *tab = s->table(t);
			Column *co =0;
			//and columns in the tables
			for (unsigned int i =0 ; i < tab->columns() ; ++i)
			{
				//Check if the column is a derive parameter value
				co = tab->column(i);
				string colName = co->name();
				std::size_t found = colName.find('D',1);

				if ( found == 1)
				{
					errstr << "Derive Parameter Column " << colName << Sas::message(Msg::App,Msg::Verbose);
					ColumnData *colData = co->data();
					switch( colData->dataType() )
					{
					case Column::Real32 :
					{
						errstr << "Real32 Derive Parameter Column " << Sas::message(Msg::App,Msg::Verbose);
						real32 *c = colData->real32Data();
						std::size_t pos = colName.find("AD082_",0,6);
						if ( (colName != "FD258") && (pos == std::string::npos) )
						{
							*s->addAttribute("DPSCORRF","Float to Double conversion corrected","") = "PATCHD";
							for (unsigned int j = 0; j < co->rows() ; ++j)
							{
								memcpy(&dest,&c[j],sizeof(double));
								int64_t * ptr = (int64_t *) (& dest) ;
								*ptr <<= 32 ; // left shift by n bits
								errstr << "Col Name: "<< colName << "[ " << j << " ] "<< " old value " << c[j] <<
										" new value "<< dest << Sas::message(Msg::App,Msg::Verbose);
								c[j] = dest;

							}
						}
						else
							errstr << "Correction not needed for "<< colName << Sas::message(Msg::App,Msg::Sparse);
					}
					break;
					default :
						errstr << "Not Real32 Derive Parameter Column "  << Sas::message(Msg::App,Msg::Verbose);
						break;
					}
					delete colData;
				}

			}
		}
	}
	else if  ( (s->hasAttribute("DPSCORRF") ) && (s->attribute("DPSCORRF")->asString() == "VALID") )
		errstr <<"The derived parameter values are correct. Nothing to do" << Sas::warning("DerivedParamCorrect");
	else if ((s->hasAttribute("DPSCORRF") ) && (s->attribute("DPSCORRF")->asString() == "PATCHD") )
	{
		errstr <<"The keyword DPSCORRF exists. Bit pattern for float derived parameter values already corrected." << Sas::warning("DerivedParametersCorrected");
	}
	else
		errstr <<"The keyword DPSCORRF exists with value: '" << s->attribute("DPSCORRF")->asString()
			<< "'. Please check derived parameter values" << Sas::error("UnknownKeywordValue");


	dataSetServer->close(s);
ENDPROC
}
