// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include <iostream>
#include <string>
#include <cstring>
#include <cstdlib> // for atof(3)
#include <bitset>
#include <iomanip>
#include <inttypes.h> // for uint32_t
#include <cassert>
#include <cmath>

using namespace std;

void floatToBinary(float, string&);
void doubleToBinary(double, string&);

int main(int argc, const char* argv[])
{
    string str;
    float f;
    if (argc > 1)
    {
        f = static_cast<float>(atof(argv[1]));
        floatToBinary(f, str);
    }
    else
        f = -1.562763;


    double d = 0.0;
    doubleToBinary(d,str);
    cout << str << endl;
    memcpy(&d,&f,sizeof(double));

    cout << "f " << f << " d " << d << endl;

    int64_t * ptr = (int64_t *) (& d) ;
    *ptr <<= 32 ; // left shift by n bits
    doubleToBinary(d,str);
    cout << str << endl;
    cout << "f " << f << " dC " << d << endl;


    double epsilon = std::abs(d- -0.187763);
    assert(epsilon < 1e-7);

    d =-0.18776298;

    doubleToBinary(d,str);
    cout << str << endl;
    cout << "f " << f << " dO " << d << endl;

    return 0;
}


void floatToBinary(float f, string& str)
{
    union { float f; uint32_t i; } u;
    u.f = f;
    str.clear();

    for (int i = 0; i < 32; i++)
    {
        if (u.i % 2)  str.push_back('1');
        else str.push_back('0');
        u.i >>= 1;
    }

    // std::bitset<sizeof(float) * CHAR_BIT>   bits(u.i);
    //
    // cout << "bitset " << bits << endl;

    // Reverse the string since now it's backwards
    string temp(str.rbegin(), str.rend());
    str = temp;
}

void doubleToBinary(double d, string& str)
{
    union { double d; uint64_t i; } u;
    u.d = d;
    str.clear();

    for (int i = 0; i < 64; i++)
    {
        if (u.i % 2)  str.push_back('1');
        else str.push_back('0');
        u.i >>= 1;
    }

    // std::bitset<sizeof(float) * CHAR_BIT>   bits(u.i);
    //
    // cout << "bitset " << bits << endl;

    // Reverse the string since now it's backwards
    string temp(str.rbegin(), str.rend());
    str = temp;
}



